@@ -13,7 +13,7 @@
  | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
  | GNU General Public License for more details.                            |
  +-------------------------------------------------------------------------+
- | Cacti: The Complete RRDTool-based Graphing Solution                     |
+ | Cacti: The Complete RRDtool-based Graphing Solution                     |
  +-------------------------------------------------------------------------+
  | This code is designed, written, and maintained by the Cacti Group. See  |
  | about.php and/or the AUTHORS file for specific developer information.   |
@@ -215,7 +215,7 @@ function load_fallback_procedure(){
 	define('CACTI_COUNTRY', 'us');
 	define('CACTI_LANGUAGE', 'English');
 	define('CACTI_LANGUAGE_FILE', 'english_usa');
-	
+
 }
 
 
@@ -241,7 +241,7 @@ function load_i18n_gettext_wrappers(){
 		if (isset($l10n[$domain])) {
     		return $l10n[$domain]->ngettext($singular, $plural, $number);
     	}else {
-			return ($number == 1) ? $singular : $plural;	
+			return ($number == 1) ? $singular : $plural;
 		}
 	}
 
@@ -257,11 +257,11 @@ function load_i18n_gettext_wrappers(){
 			return false;
 
 		/* convert pure text strings */
-		}elseif ($num == 1) {
+		} elseif ($num == 1) {
 			return __gettext($args[0]);
 
 		/* convert pure text strings by using a different textdomain */
-		}elseif ($num == 2 && isset($l10n[$args[1]])) {
+		} elseif ($num == 2 && isset($l10n[$args[1]])) {
 			return __gettext($args[0], $args[1]);
 
 		/* convert stings including one or more placeholders */
@@ -282,7 +282,7 @@ function load_i18n_gettext_wrappers(){
 	function __xn($context, $singular, $plural, $number, $domain = 'cacti'){
 		$xsingular = $context . chr(4) . $singular;
 		$xplural = $context . chr(4) . $plural;
-		
+
 		$msgstr = __n($xsingular, $xplural, $number, $domain);
 		if($number == 1 ) {
 			return ( $msgstr == $xsingular ) ? $singular : $msgstr;
@@ -290,40 +290,42 @@ function load_i18n_gettext_wrappers(){
 			return ( $msgstr == $xplural ) ? $plural : $msgstr;
 		}
 	}
-	
+
 	function __x(){
+		global $l10n;
+
 		$args = func_get_args();
 		$num  = func_num_args();
-		
+
 		/* this should never happen */
 		if ($num < 2) {
 			return false;
 		}else {
 			$context = array_shift($args);
 			$num--;
-			
+
 			$msgid = reset($args);
 			$xmsgid = $context . chr(4) . $msgid;
-			
+
 			$args[0] = $xmsgid;
-			
-			if($num == 2) {
+
+			if($num == 1) {
 				/* pure text string without placeholders and a change of the default textdomain */
 				$msgstr = __gettext($args[0]);
 			}else {
 				/* get gettext string */
 				$msgstr = isset($l10n[$args[$num-1]]) 	? __gettext($args[0], $args[$num-1])
-														: __gettext($args[0]);			
+														: __gettext($args[0]);
 			}
-			
+
 			/* use the raw message id if language catalogue does not contain a context specific message string */
 			$args[0] = ( $msgstr == $xmsgid ) ? $msgid : $msgstr;
-			
+
 			/* process return string against input arguments */
 			return call_user_func_array('sprintf', $args);
 		}
 	}
-	
+
 	function __date($format, $timestamp = false, $domain = 'cacti') {
 
 		global $i18n_date_placeholders;
@@ -384,11 +386,11 @@ function load_i18n_fallback_wrappers(){
 			return false;
 
 		/* convert pure text strings */
-		}elseif ($num == 1) {
+		} elseif ($num == 1) {
 			return $args[0];
 
 		/* convert pure text strings by using a different textdomain */
-		}elseif ($num == 2 && isset($l10n[$args[1]])) {
+		} elseif ($num == 2 && isset($l10n[$args[1]])) {
 			return $args[0];
 
 		/* convert stings including one or more placeholders */
@@ -405,11 +407,11 @@ function load_i18n_fallback_wrappers(){
 	function __xn($context, $singular, $plural, $number, $domain = 'cacti'){
 		return __n($singular, $plural, $number, $domain);
 	}
-	
+
 	function __x(){
 		$args = func_get_args();
 		$num  = func_num_args();
-		
+
 		/* this should never happen */
 		if ($num < 2) {
 			return false;
@@ -425,8 +427,6 @@ function load_i18n_fallback_wrappers(){
 	}
 }
 
-
-
 /**
  * get_list_of_locales - returns the default settings being used for l10n
  *
@@ -434,55 +434,56 @@ function load_i18n_fallback_wrappers(){
  */
 function get_list_of_locales () {
 	$lang2locale = array(
-		'sq'	=> array('language' => 'Albanian',				'country' => 'al', 'filename' => 'albanian_albania'),
-		'ar'	=> array('language' => 'Arabic',				'country' => 'sa', 'filename' => 'arabic_saudi_arabia'),
-		'hy'	=> array('language' => 'Armenian',				'country' => 'am', 'filename' => 'armenian_armenia'),
-		'be'	=> array('language' => 'Belarusian',			'country' => 'by', 'filename' => 'belarusian_belarus'),
-		'bg'	=> array('language' => 'Bulgarian',			'country' => 'bg', 'filename' => 'bulgarian_bulgaria'),
-		'zh'	=> array('language' => 'Chinese',				'country' => 'cn', 'filename' => 'chinese_china'),
-		'zh-cn'=> array('language' => 'Chinese (China)',			'country' => 'cn', 'filename' => 'chinese_china_simplified'),
-		'zh-hk'=> array('language' => 'Chinese (Hong Kong)',		'country' => 'hk', 'filename' => 'chinese_hong_kong'),
-		'zh-sg'=> array('language' => 'Chinese (Singapore)',		'country' => 'sg', 'filename' => 'chinese_singapore'),
-		'zh-tw'=> array('language' => 'Chinese (Taiwan)',		'country' => 'tw', 'filename' => 'chinese_taiwan'),
-		'hr'	=> array('language' => 'Croatian',				'country' => 'hr', 'filename' => 'croatian_croatia'),
-		'cs'	=> array('language' => 'Czech',				'country' => 'cz', 'filename' => 'czech_czech_republic'),
-		'da'	=> array('language' => 'Danish',				'country' => 'dk', 'filename' => 'danish_denmark'),
-		'nl'	=> array('language' => 'Dutch',				'country' => 'nl', 'filename' => 'dutch_netherlands'),
-		'en'	=> array('language' => 'English',				'country' => 'us', 'filename' => 'english_usa'),
-		'et'	=> array('language' => 'Estonian',				'country' => 'ee', 'filename' => 'estonian_estonia'),
-		'fi'	=> array('language' => 'Finnish',				'country' => 'fi', 'filename' => 'finnish_finland'),
-		'fr'	=> array('language' => 'French',				'country' => 'fr', 'filename' => 'french_france'),
-		'de'	=> array('language' => 'German',				'country' => 'de', 'filename' => 'german_germany'),
-		'el'	=> array('language' => 'Greek',				'country' => 'gr', 'filename' => 'greek_greece'),
-		'iw'	=> array('language' => 'Hebrew',				'country' => 'il', 'filename' => 'hebrew_israel'),
-		'hi'	=> array('language' => 'Hindi',				'country' => 'in', 'filename' => 'hindi_india'),
-		'hu'	=> array('language' => 'Hungarian',			'country' => 'hu', 'filename' => 'hungarian_hungary'),
-		'is'	=> array('language' => 'Icelandic',			'country' => 'is', 'filename' => 'icelandic_iceland'),
-		'id'	=> array('language' => 'Indonesian',			'country' => 'id', 'filename' => 'indonesian_indonesia'),
-		'ga'	=> array('language' => 'Irish',				'country' => 'ie', 'filename' => 'irish_ireland'),
-		'it'	=> array('language' => 'Italian',				'country' => 'it', 'filename' => 'italian_italy'),
-		'ja'	=> array('language' => 'Japanese',				'country' => 'jp', 'filename' => 'japanese_japan'),
-		'ko'	=> array('language' => 'Korean',				'country' => 'kr', 'filename' => 'korean_korea'),
-		'lv'	=> array('language' => 'Lativan',				'country' => 'lv', 'filename' => 'latvian_latvia'),
-		'lt'	=> array('language' => 'Lithuanian',			'country' => 'lt', 'filename' => 'lithuanian_lithuania'),
-		'mk'	=> array('language' => 'Macedonian',			'country' => 'mk', 'filename' => 'macedonian_macedonia'),
-		'ms'	=> array('language' => 'Malay',				'country' => 'my', 'filename' => 'malay_malaysia'),
-		'mt'	=> array('language' => 'Maltese',				'country' => 'lt', 'filename' => 'maltese_malta'),
-		'no'	=> array('language' => 'Norwegian',			'country' => 'no', 'filename' => 'norwegian_norway'),
-		'pl'	=> array('language' => 'Polish',				'country' => 'pl', 'filename' => 'polish_poland'),
-		'pt'	=> array('language' => 'Portuguese',			'country' => 'pt', 'filename' => 'portuguese_portugal'),
-		'pt-br'=> array('language' => 'Portuguese (Brazil)',		'country' => 'br', 'filename' => 'portuguese_brazil'),
-		'ro'	=> array('language' => 'Romanian',				'country' => 'ro', 'filename' => 'romanian_romania'),
-		'ru'	=> array('language' => 'Russian',				'country' => 'ru', 'filename' => 'russian_russia'),
-		'sr'	=> array('language' => 'Serbian',				'country' => 'rs', 'filename' => 'serbian_serbia'),
-		'sk'	=> array('language' => 'Slovak',				'country' => 'sk', 'filename' => 'slovak_slovakia'),
-		'sl'	=> array('language' => 'Slovenian',			'country' => 'si', 'filename' => 'slovenian_slovenia'),
-		'es'	=> array('language' => 'Spanish',				'country' => 'es', 'filename' => 'spanish_spain'),
-		'sv'	=> array('language' => 'Swedish',				'country' => 'se', 'filename' => 'swedish_sweden'),
-		'th'	=> array('language' => 'Thai',				'country' => 'th', 'filename' => 'thai_thailand'),
-		'tr'	=> array('language' => 'Turkish',				'country' => 'tr', 'filename' => 'turkish_turkey'),
-		'vi'	=> array('language' => 'Vietnamese',			'country' => 'vn', 'filename' => 'vietnamese_vietnam')
+		'sq'	=> array('language' => 'Albanian',            'country' => 'al', 'filename' => 'albanian_albania'),
+		'ar'	=> array('language' => 'Arabic',              'country' => 'sa', 'filename' => 'arabic_saudi_arabia'),
+		'hy'	=> array('language' => 'Armenian',            'country' => 'am', 'filename' => 'armenian_armenia'),
+		'be'	=> array('language' => 'Belarusian',          'country' => 'by', 'filename' => 'belarusian_belarus'),
+		'bg'	=> array('language' => 'Bulgarian',           'country' => 'bg', 'filename' => 'bulgarian_bulgaria'),
+		'zh'	=> array('language' => 'Chinese',             'country' => 'cn', 'filename' => 'chinese_china'),
+		'zh-cn' => array('language' => 'Chinese (China)',     'country' => 'cn', 'filename' => 'chinese_china_simplified'),
+		'zh-hk' => array('language' => 'Chinese (Hong Kong)', 'country' => 'hk', 'filename' => 'chinese_hong_kong'),
+		'zh-sg' => array('language' => 'Chinese (Singapore)', 'country' => 'sg', 'filename' => 'chinese_singapore'),
+		'zh-tw' => array('language' => 'Chinese (Taiwan)',    'country' => 'tw', 'filename' => 'chinese_taiwan'),
+		'hr'	=> array('language' => 'Croatian',            'country' => 'hr', 'filename' => 'croatian_croatia'),
+		'cs'	=> array('language' => 'Czech',               'country' => 'cz', 'filename' => 'czech_czech_republic'),
+		'da'	=> array('language' => 'Danish',              'country' => 'dk', 'filename' => 'danish_denmark'),
+		'nl'	=> array('language' => 'Dutch',               'country' => 'nl', 'filename' => 'dutch_netherlands'),
+		'en'	=> array('language' => 'English',             'country' => 'us', 'filename' => 'english_usa'),
+		'et'	=> array('language' => 'Estonian',            'country' => 'ee', 'filename' => 'estonian_estonia'),
+		'fi'	=> array('language' => 'Finnish',             'country' => 'fi', 'filename' => 'finnish_finland'),
+		'fr'	=> array('language' => 'French',              'country' => 'fr', 'filename' => 'french_france'),
+		'de'	=> array('language' => 'German',              'country' => 'de', 'filename' => 'german_germany'),
+		'el'	=> array('language' => 'Greek',               'country' => 'gr', 'filename' => 'greek_greece'),
+		'iw'	=> array('language' => 'Hebrew',              'country' => 'il', 'filename' => 'hebrew_israel'),
+		'hi'	=> array('language' => 'Hindi',               'country' => 'in', 'filename' => 'hindi_india'),
+		'hu'	=> array('language' => 'Hungarian',           'country' => 'hu', 'filename' => 'hungarian_hungary'),
+		'is'	=> array('language' => 'Icelandic',           'country' => 'is', 'filename' => 'icelandic_iceland'),
+		'id'	=> array('language' => 'Indonesian',          'country' => 'id', 'filename' => 'indonesian_indonesia'),
+		'ga'	=> array('language' => 'Irish',               'country' => 'ie', 'filename' => 'irish_ireland'),
+		'it'	=> array('language' => 'Italian',             'country' => 'it', 'filename' => 'italian_italy'),
+		'ja'	=> array('language' => 'Japanese',            'country' => 'jp', 'filename' => 'japanese_japan'),
+		'ko'	=> array('language' => 'Korean',              'country' => 'kr', 'filename' => 'korean_korea'),
+		'lv'	=> array('language' => 'Lativan',             'country' => 'lv', 'filename' => 'latvian_latvia'),
+		'lt'	=> array('language' => 'Lithuanian',          'country' => 'lt', 'filename' => 'lithuanian_lithuania'),
+		'mk'	=> array('language' => 'Macedonian',          'country' => 'mk', 'filename' => 'macedonian_macedonia'),
+		'ms'	=> array('language' => 'Malay',               'country' => 'my', 'filename' => 'malay_malaysia'),
+		'mt'	=> array('language' => 'Maltese',             'country' => 'lt', 'filename' => 'maltese_malta'),
+		'no'	=> array('language' => 'Norwegian',           'country' => 'no', 'filename' => 'norwegian_norway'),
+		'pl'	=> array('language' => 'Polish',              'country' => 'pl', 'filename' => 'polish_poland'),
+		'pt'	=> array('language' => 'Portuguese',          'country' => 'pt', 'filename' => 'portuguese_portugal'),
+		'pt-br' => array('language' => 'Portuguese (Brazil)', 'country' => 'br', 'filename' => 'portuguese_brazil'),
+		'ro'	=> array('language' => 'Romanian',            'country' => 'ro', 'filename' => 'romanian_romania'),
+		'ru'	=> array('language' => 'Russian',             'country' => 'ru', 'filename' => 'russian_russia'),
+		'sr'	=> array('language' => 'Serbian',             'country' => 'rs', 'filename' => 'serbian_serbia'),
+		'sk'	=> array('language' => 'Slovak',              'country' => 'sk', 'filename' => 'slovak_slovakia'),
+		'sl'	=> array('language' => 'Slovenian',           'country' => 'si', 'filename' => 'slovenian_slovenia'),
+		'es'	=> array('language' => 'Spanish',             'country' => 'es', 'filename' => 'spanish_spain'),
+		'sv'	=> array('language' => 'Swedish',             'country' => 'se', 'filename' => 'swedish_sweden'),
+		'th'	=> array('language' => 'Thai',                'country' => 'th', 'filename' => 'thai_thailand'),
+		'tr'	=> array('language' => 'Turkish',             'country' => 'tr', 'filename' => 'turkish_turkey'),
+		'vi'	=> array('language' => 'Vietnamese',          'country' => 'vn', 'filename' => 'vietnamese_vietnam')
 	);
+
 	return $lang2locale;
 }
 
@@ -512,6 +513,8 @@ function get_installed_locales(){
 		}
 	}
 
+	asort($supported_languages);
+
 	return $supported_languages;
 }
 
@@ -525,8 +528,6 @@ function read_user_i18n_setting($config_name) {
 	/* users must have cacti user auth turned on to use this, or the guest account must be active */
 	if (isset($_SESSION['sess_user_id'])) {
 		$effective_uid = $_SESSION['sess_user_id'];
-	}else if (isset($config['config_options_array']['export_user_id'])) {
-		$effective_uid = $config['config_options_array']['export_user_id'];
 	}else if ((read_config_option('auth_method') == 0)) {
 		if (isset($_SESSION['sess_config_array'])) {
 			$config_array = $_SESSION['sess_config_array'];
@@ -534,30 +535,30 @@ function read_user_i18n_setting($config_name) {
 			$config_array = $config['config_options_array'];
 		}
 		if (!isset($config_array[$config_name])) {
-			$effective_uid = db_fetch_cell("SELECT user_auth.id 
-				FROM settings 
-				INNER JOIN user_auth 
-				ON user_auth.username = settings.value 
+			$effective_uid = db_fetch_cell("SELECT user_auth.id
+				FROM settings
+				INNER JOIN user_auth
+				ON user_auth.username = settings.value
 				WHERE settings.name = 'guest_user'");
 		}
-		if (strlen($effective_uid) == 0) {
+		if ($effective_uid == '') {
 			$effective_uid = 0;
 		}
-	}else{
+	} else {
 		$effective_uid = 0;
 	}
 
 	if (db_table_exists('settings_user')) {
-		$db_setting = db_fetch_row_prepared('SELECT value 
-			FROM settings_user 
-			WHERE name = ? 
-			AND user_id = ?', 
+		$db_setting = db_fetch_row_prepared('SELECT value
+			FROM settings_user
+			WHERE name = ?
+			AND user_id = ?',
 			array($config_name, $effective_uid));
 	}
-	
+
 	if (isset($db_setting['value'])) {
 		return $db_setting['value'];
-	}else{
+	} else {
 		return false;
 	}
 }
@@ -578,14 +579,14 @@ function number_format_i18n($number, $decimals = 0, $baseu = 1024) {
 		numfmt_set_attribute($fmt, NumberFormatter::MAX_FRACTION_DIGITS, $decimals);
 
 		return numfmt_format($fmt, $number);
-	}else{
+	} else {
 		$origlocales = explode(';', setlocale(LC_ALL, 0));
 		setlocale(LC_ALL, $cacti_locale . '_' . $country);
 		$locale = localeconv();
 
 		if ($decimals == -1) {
 			$number =  number_format($number, $decimals, $locale['decimal_point'], $locale['thousands_sep']);
-		}elseif ($number>=pow($baseu, 4)) {
+		} elseif ($number>=pow($baseu, 4)) {
 			$number =  number_format($number/pow($baseu, 4), $decimals, $locale['decimal_point'], $locale['thousands_sep']) . __(' T');
 		} elseif($number>=pow($baseu, 3)) {
 			$number = number_format($number/pow($baseu, 3), $decimals, $locale['decimal_point'], $locale['thousands_sep']) . __(' G');
