@@ -274,7 +274,7 @@ function form_actions() {
 
 	form_start('pollers.php');
 
-	html_start_box($poller_actions{get_nfilter_request_var('drp_action')}, '60%', '', '3', 'center', '');
+	html_start_box($poller_actions[get_nfilter_request_var('drp_action')], '60%', '', '3', 'center', '');
 
 	if (isset($poller_array) && sizeof($poller_array)) {
 		if (get_nfilter_request_var('drp_action') == '1') { // delete
